<?php

/**
 * This file contains QUI\ERP\Employee\EventHandler
 */

namespace QUI\ERP\Employee;

use QUI;
use QUI\Exception;
use QUI\Package\Package;

/**
 * Class EventHandler
 *
 * @package QUI\ERP
 */
class EventHandler
{
    /**
     * event: on package setup
     * - create employee group
     *
     * @param Package $Package
     * @throws Exception
     */
    public static function onPackageSetup(Package $Package): void
    {
        if ($Package->getName() != 'quiqqer/employee') {
            return;
        }

        // create employee group
        $Config = $Package->getConfig();
        $groupId = $Config->getValue('employeeGeneral', 'groupId');

        // downward compatibility
        if (empty($groupId) && $Config->getValue('general', 'groupId')) {
            $groupId = $Config->getValue('general', 'groupId');
            $advisorId = $Config->getValue('general', 'advisorId');
            $employeeLogin = $Config->getValue('general', 'employeeLogin');

            $Config->setValue('employeeGeneral', 'groupId', $groupId);

            if (!empty($advisorId)) {
                $Config->setValue('employeeGeneral', 'advisorId', $advisorId);
            }

            if (!empty($employeeLogin)) {
                $Config->setValue('employeeGeneral', 'employeeLogin', $employeeLogin);
            }

            $Config->del('general');
            $Config->save();
        }

        if (!empty($groupId)) {
            return;
        }

        $Root = QUI::getGroups()->firstChild();

        $Employee = $Root->createChild(
            QUI::getLocale()->get('quiqqer/employee', 'employee.group.name'),
            QUI::getUsers()->getSystemUser()
        );

        $Config->setValue('employeeGeneral', 'groupId', $Employee->getUUID());
        $Config->save();
    }

    /**
     * event : on admin header loaded
     */
    public static function onAdminLoadFooter(): void
    {
        if (!defined('ADMIN') || !ADMIN) {
            return;
        }

        $Package = QUI::getPackageManager()->getInstalledPackage('quiqqer/employee');
        $Config = $Package->getConfig();
        $groupId = $Config->getValue('employeeGeneral', 'groupId');

        if (!$groupId) {
            $groupId = 0;
        }

        echo '<script>window.QUIQQER_EMPLOYEE_GROUP = "' . $groupId . '";</script>';
    }
}
